// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// AccelParticle.java
// The particles produced by the agents when they accelerate
// ------------------------------------------------------------------------------

class AccelParticle extends Particle
{
  static final float maxRotationDeviation = 40;
  static final float maxLifespan = 30;
  static final float initialRadius = 2.5;
  float r1;
  float g1;
  float b1;
  float r2;
  float g2;
  float b2;
  float rotationDeviation;
  int lifespan;

  /**
   * Constructor
   */
  AccelParticle(float R, Point C, Vector V)
  {
    super(R, C, V, false);
    
    rotationDeviation = radians(random(-maxRotationDeviation/2, maxRotationDeviation/2));
    veloc.set((1+random(-0.5,0.5))*veloc.length(), veloc.dir() + rotationDeviation);
  }

  /**
   * Updates the particles position
   */
  void frame()
  {
    lifespan++;

    center = veloc.add(center);

    if (lifespan > maxLifespan)
    {
      die();
    }
  }

  /**
   * Draws the particle
   */
  void render(float x, float y)
  {
    float r = r2 - 2*abs((r1-r2)*rotationDeviation/radians(maxRotationDeviation));
    float g = g2 - 2*abs((g1-g2)*rotationDeviation/radians(maxRotationDeviation));
    float b = b2 - 2*abs((b1-b2)*rotationDeviation/radians(maxRotationDeviation));
    float a = 255 - 255*lifespan/maxLifespan;
    float s = 2*(initialRadius - initialRadius*lifespan/maxLifespan);

    noStroke();
    fill(r, g, b, a);
    ellipse(x, y, s, s);
  }
}
