import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class Arkigen extends PApplet {// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// AngularV2.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// The instance of the 'Game' class
Game game;

/**
 * Only gets executed once upon startup
 */
public void setup() 
{
   size(800,420);
   smooth();
   frameRate(30);
   game = new Game();
}

/**
 * Gets executed on each frame
 */
public void draw() 
{
  // Execute one frame of the game
  game.frame();
}

/**
 * Mousepressed listener. It tells the game
 * that the user has pressed the mouse button
 */
public void mousePressed()
{
  game.mousePressed();
}

/**
 * Mousedragged listener. It tells the game
 * that the user has dragged the mouse
 */
public void mouseDragged()
{
  game.mouseDragged();
}

/**
 * Keypressed listener. It tells the game
 * that the user has pressed a key
 */
public void keyPressed()
{
  game.keyPressed();
}

/**
 * Keyreleased listener. It tells the game
 * that the user has released a key
 */
public void keyReleased()
{
  game.keyReleased();
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// AccelHumanParticle.java
// The particles produced by the _human_ agents when they accelerate
// ------------------------------------------------------------------------------

class AccelHumanParticle extends AccelParticle
{
  AccelHumanParticle(float R, Point C, Vector V)
  {
    super(R, C, V);
    
    r1 = 237;
    g1 = 27;
    b1 = 35;
    r2 = 247;
    g2 = 244;
    b2 = 28;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// AccelParticle.java
// The particles produced by the agents when they accelerate
// ------------------------------------------------------------------------------

class AccelParticle extends Particle
{
  static final float maxRotationDeviation = 40;
  static final float maxLifespan = 30;
  static final float initialRadius = 2.5f;
  float r1;
  float g1;
  float b1;
  float r2;
  float g2;
  float b2;
  float rotationDeviation;
  int lifespan;

  /**
   * Constructor
   */
  AccelParticle(float R, Point C, Vector V)
  {
    super(R, C, V, false);
    
    rotationDeviation = radians(random(-maxRotationDeviation/2, maxRotationDeviation/2));
    veloc.set((1+random(-0.5f,0.5f))*veloc.length(), veloc.dir() + rotationDeviation);
  }

  /**
   * Updates the particles position
   */
  public void frame()
  {
    lifespan++;

    center = veloc.add(center);

    if (lifespan > maxLifespan)
    {
      die();
    }
  }

  /**
   * Draws the particle
   */
  public void render(float x, float y)
  {
    float r = r2 - 2*abs((r1-r2)*rotationDeviation/radians(maxRotationDeviation));
    float g = g2 - 2*abs((g1-g2)*rotationDeviation/radians(maxRotationDeviation));
    float b = b2 - 2*abs((b1-b2)*rotationDeviation/radians(maxRotationDeviation));
    float a = 255 - 255*lifespan/maxLifespan;
    float s = 2*(initialRadius - initialRadius*lifespan/maxLifespan);

    noStroke();
    fill(r, g, b, a);
    ellipse(x, y, s, s);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Agent.java
// The main characters in the game. This class will eventuallyl be inherited 
// by the human or computer agent classes.
// ------------------------------------------------------------------------------

class Agent extends PhysicsModel
{
  float accelK;
  float turnK;
  float brakeK;
  float dampK;
  float smallestModelRadius;

  /**
   * Constructor
   */
  Agent(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Agent", P, M, C, R, S, I);
  }
  
  /**
   * Moves the agent according to the input variables
   */
  public void drive(boolean accelerate, boolean brake, boolean turnLeft, boolean turnRight, String type)
  {
    Point c = new Point(0,0);                                // Center position
    Point b = new Point(0,8).rotate(poly.rotation);          // Rear exhaust position
    Point f = new Point(0,-8).rotate(poly.rotation);         // Front tire position
    Point r = new Point(-5,8).rotate(poly.rotation);         // Right wing tip
    Point l = new Point(+5,8).rotate(poly.rotation);         // Left wing tip
    
    Force cfNormal = new Force(c, new Vector(new Point(0,-1).rotate(poly.rotation)));       // Acceleration force with length 1
    Force ffNormal = new Force(f, new Vector(new Point(-1,0).rotate(poly.rotation)));       // Front steering left force with length 1
    
    if (accelerate)
    {
      forces.add(cfNormal.multiply(accelK));
      emitParticles("Accel" + type, 0, new Vector(center).add(b), cfNormal.vector.multiply(-2), 4);
    }
    if (brake)
    {
      forces.add(cfNormal.multiply(-brakeK));
    }
    if (turnLeft)
    {
      forces.add(ffNormal.multiply(turnK));
    }
    if (turnRight)
    {
      forces.add(ffNormal.multiply(-turnK));
    }
  }
  
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {     
    String[] args = inputToArray(input);
     
    accelK = Str2Int(args[0]);
    turnK = Str2Flt(args[1]);
    brakeK = Str2Flt(args[2]);
    dampK = Str2Flt(args[3]);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Camera.java
// Agent that controls the movements of the camera.
// ------------------------------------------------------------------------------

class Camera extends Model
{
  int follow;
  float invSpeed;
  float maxDisp;
  
  /**
   * Constructor
   */
  Camera(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Camera", P, M, C, R, S, I);
  }
  
   /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    if (invSpeed != 0)
    {
      Model followModel = (Model) models.get(follow);
      Vector difference = new Vector(followModel.center.x - center.x, followModel.center.y - center.y);
      Point targetCenter = difference.multiply(1/invSpeed).add(center);
    
      if (new Vector(targetCenter).length() > maxDisp)
      {
        float theta = atan2(targetCenter.y, targetCenter.x);
        float x = maxDisp*cos(theta);  // Can't use TrigTable because it 
        float y = maxDisp*sin(theta);  // isn't smooth enough for the camera
        center = new Point(x,y);
      }
      else
      {
        center = targetCenter;
      }
    }
    
    return this;
  }
  
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {     
    String[] args = inputToArray(input);
     
    follow = Str2Int(args[0]);
    invSpeed = Str2Flt(args[1]);
    maxDisp = Str2Flt(args[2]);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// DataLoader.java
// This class will make loading data from the disk easy.
// ------------------------------------------------------------------------------

class DataLoader
{
  /**
   * Load an nxn matrix from disk
   */
  public String[][] loadData(String location, String separator)
  {
    String[] data = loadStrings(location);
    
    if (data.length != 0)
    {
      String[] tokens = data[0].split(separator);
    
      // Assumes that each line has the same number of tokens
      String[][] R = new String[data.length][tokens.length];
    
      // Fill up R.
      for (int i=0; i<data.length; i++)
      {
        for (int j=0; j<tokens.length; j++)
        {
          R[i][j] = data[i].split(separator)[j];
        }
      }
      
      return R;
    }
    else
    {
      return new String[0][0];
    }
  }
  
  /**
   * Load an nx1 matrix from disk
   */
  public String[] loadData(String location)
  {
    String[] data = loadStrings(location);
    
    // Assumes that each line has the same number of tokens
    String[] R = new String[data.length];
    
    // Fill up R.
    for (int i=0; i<data.length; i++)
    {
        R[i] = data[i];
    }
    
    return R;
  }
  
  /**
   * Preset function that loads polygon information
   * from disk
   */
  public Polygon loadPolygon(String P)
  {
    // The polygon that gets returned
    Polygon R = new Polygon();
    
    // Each line represents one point in the polygon
    String[][] polyData = loadData("data/polygons/" + P + ".aip", ",");
    for (int j=0; j<polyData.length; j++)
    {
      float x = Str2Flt(polyData[j][0]);
      float y = Str2Flt(polyData[j][1]);
      R.insertVertex(new Point(x,y));
    }
    
    return R;
  }
  
  /**
   * Preset function that loads material information
   * from disk
   */
  public Material loadMaterial(String M)
  {
    // The material type that gets returned
    Material R;
        
    // modelData[0] = Specular
    // modelData[1] = Glow
    // modelData[2-5] = Body color
    // modelData[6-8] = Border color
    // modelData[9] = Stroke Weight
    // modelData[10] = Inverse Density
    // modelData[11] = Bounce constant
    String[] modelData = loadData("data/materials/" + M + ".aim");
    int body = color(Str2Int(modelData[2]), Str2Int(modelData[3]), Str2Int(modelData[4]), Str2Flt(modelData[5]));
    int border = color(Str2Int(modelData[6]), Str2Int(modelData[7]), Str2Int(modelData[8]));
    R = new Material(Str2Flt(modelData[0]), Str2Flt(modelData[1]), color(Str2Int(modelData[2]), Str2Int(modelData[3]), Str2Int(modelData[4]), Str2Flt(modelData[5])), color(Str2Int(modelData[6]), Str2Int(modelData[7]), Str2Int(modelData[8])), Str2Int(modelData[9]), Str2Flt(modelData[10]), Str2Flt(modelData[11])/100);
    
    return R;
  }
  
  /**
   * Convert a String to an Integer
   */
  public int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  public float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Drag.java
// Simple agent that just follows the mouse.
// ------------------------------------------------------------------------------

class Drag extends Model
{
  float invSpeed;
   
  /**
   * Constructor
   */
  Drag(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Drag", P, M, C, R, S, I);
  }
   
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    Model camera = findCamera(models);
    Vector difference = new Vector((mouseX - width/2)/settings.PIXELS_PER_METER - center.x + camera.center.x, (mouseY - height/2)/settings.PIXELS_PER_METER - center.y + camera.center.y);
    center = difference.multiply(1/invSpeed).add(center);
    return this;
  }
  
  /**
   * Parse the input string. This agent doesn't
   * take in any arguments.
   */
  public void parseInput(String input)
  {
    String[] args = inputToArray(input);
    
    invSpeed = Str2Flt(args[0]);
  }
  
  /**
  * Finds the camera in the list of models
  */
  public Model findCamera(ArrayList models)
  {
    return (Model) models.get(0);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Force.java
// Data type for forces. Forces are vectors with points of application.
// ------------------------------------------------------------------------------

class Force
{
  Point point;
  Vector vector;
  
  /**
   * Constructor
   */
  Force()
  {
    point = new Point();
    vector = new Vector();
  }
  
  /**
   * Constructor with arguments
   */
  Force(Point P, Vector V)
  {
    point = P;
    vector = V;
  }
  
  /**
   * Multiply force
   */
  public Force multiply(float k)
  {
    Force R = new Force();
    R.point = point;
    R.vector = vector.multiply(k);
    return R;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Game.java
// The game class. It will handle the user's inputs, run the AI, display graphics
// and basically do everything that isn't related to Processing's internal
// functions in AI.java
// ------------------------------------------------------------------------------

class Game
{
  Settings settings;              // Game settings loaded from the settings file
  GameWorld currentWorld;         // The actual game. Everything dynamic happens here
  Renderer renderer;              // Draws everything onto the screen
  boolean[] keys;                 // Keeps track of the keys that are pressed
  
  /**
   * Constructor
   */
  Game()
  {
    // Load settings
    settings = new Settings("settings.ais");
    
    // Load game world
    currentWorld = loadWorld("mainworld");
    
    // Initiate renderer
    renderer = new Renderer(settings);
    
    // Initiate keys
    keys = new boolean[5];
  }
  
  /**
   * Does everything the game should do
   * in one frame. Handle AI, display graphics
   * and so on.
   */
  public void frame()
  {
    currentWorld.frame(keys);
    renderer.frame(currentWorld);
  }  
  
  /**
   * Handles mouse clicks
   */
  public void mousePressed()
  {
  }
  
  /**
   * Handles mouse drags
   */
  public void mouseDragged()
  {
  }
  
  /**
   * Handles key presses
   */
  public void keyPressed()
  {
    if (keyCode == UP)
    {
      keys[0] = true;
    }
    if (keyCode == DOWN)
    {
      keys[1] = true;
    }
    if (keyCode == LEFT)
    {
      keys[2] = true;
    }
    if (keyCode == RIGHT)
    {
      keys[3] = true;
    }   
  }
  
  /**
   * Handles key releases
   */
  public void keyReleased()
  {
    if (keyCode == UP)
    {
      keys[0] = false;
    }
    if (keyCode == DOWN)
    {
      keys[1] = false;
    }
    if (keyCode == LEFT)
    {
      keys[2] = false;
    }
    if (keyCode == RIGHT)
    {
      keys[3] = false;
    }  
  }
  
  /**
   * Loads a world and starts playing it.
   */
  public GameWorld loadWorld(String worldName)
  {
    // The world that gets returned
    GameWorld R = new GameWorld();
    
    // An instance of the DataLoader class
    DataLoader dataLoader = new DataLoader();
    
    // Load gameworld data
    String[][] gameWorldData = dataLoader.loadData(settings.GAMEWORLD_PATH + worldName + ".aiw", "\\.");
    
    for (int i=0; i<gameWorldData.length; i++)
    {
      // gameWorldData[i][0]: Model name
      // gameWorldData[i][1]: Model class
      // gameWorldData[i][2]: Model polygon type
      // gameWorldData[i][3]: Model material
      // gameWorldData[i][4]: Model x position
      // gameWorldData[i][5]: Model y position
      // gameWorldData[i][6]: Model rotation
      // gameWorldData[i][7]: Model arguments
      
      // Load the model's polygon
      Polygon polygon = dataLoader.loadPolygon(gameWorldData[i][2]);
      
      // Load the model's material
      Material material = dataLoader.loadMaterial(gameWorldData[i][3]);
      
      // Create the resulting model and
      // add it to the gameworld.
      if (gameWorldData[i][1].equals("camera"))
      {
        R.addModel(new Camera(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
      if (gameWorldData[i][1].equals("drag"))
      {
        R.addModel(new Drag(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
      if (gameWorldData[i][1].equals("static"))
      {
        R.addModel(new Static(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
      if (gameWorldData[i][1].equals("humanagent"))
      {
        R.addModel(new HumanAgent(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
      if (gameWorldData[i][1].equals("sheepdispenser"))
      {
        R.addModel(new SheepDispenser(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
      if (gameWorldData[i][1].equals("sheep"))
      {
        R.addModel(new Sheep(gameWorldData[i][0], polygon, material, new Point(Str2Int(gameWorldData[i][4]), Str2Int(gameWorldData[i][5])), radians(Str2Int(gameWorldData[i][6])), settings, gameWorldData[i][7]));
      }
    }
    
    return R;
  }
  
  /**
   * Convert a String to an Integer
   */
  public int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  public float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// GameWorld.java
// The world. This will store all the models, update the models and so on.
// ------------------------------------------------------------------------------

class GameWorld
{
  ArrayList models;  
  
  /**
   * Constructor
   */
  GameWorld()
  {
    models = new ArrayList();
  }
  
  /**
   * Adds a model to the game world
   */
  public void addModel(Model M)
  {
    models.add(M);
  }
  
  /**
   * Executes one frame of the game world
   */
  public void frame(boolean[] keys)
  {
    for (int i=0; i<models.size(); i++)
    {
      // Update each model
      Model modelI = ((Model) models.get(i)).frame(models, keys);
      models.set(i, modelI);
      
      // Ask each model if it wants to
      // add anything to the world
      Model modelToBeAdded = modelI.modelToBeAdded();
      if (modelToBeAdded != null) addModel(modelToBeAdded);      
      
      if (modelI instanceof PhysicsModel)
      {
        for (int j=i-1; j>=0; j--)
        {
          Model modelJ = (Model) models.get(j);
          
          if (modelJ instanceof PhysicsModel)
          {
            collide((PhysicsModel) modelI, (PhysicsModel) modelJ);
          }
        }
      }
    }
  }
  
  /**
   * Detects and handles collisions
   * between two models
   */
  public void collide(PhysicsModel A, PhysicsModel B)
  {
    float delta_x = A.center.x - B.center.x;
    float delta_y = A.center.y - B.center.y;
    float delta_d = sqrt(pow(delta_x,2)+pow(delta_y,2));
    float sum_r = A.poly.radius + B.poly.radius;
    
    if ((A.material.invDensity != 0 || B.material.invDensity != 0) && delta_d <= sum_r && (A.veloc.length() > 0 || B.veloc.length() > 0))
    {
      Vector MTD = calculateMTD(A, B);
      
      if (MTD.length() != 0)
      {
        Point[] contactPoints = findContactPoints(A, B, MTD);
        
        Vector vA = A.veloc;
        Vector vB = B.veloc;
        Vector vAB = vA.add(vB.multiply(-1));
        Vector rA = new Vector(contactPoints[0]);
        Vector rB = new Vector(contactPoints[1]);        
        
        float e = A.material.bounce*B.material.bounce;
        float invMassA = 1/A.mass;
        float invMassB = 1/B.mass;
        
        Vector vtA = A.veloc.add(rA.multiply(A.rotationSpeed));
        Vector vtB = B.veloc.add(rB.multiply(B.rotationSpeed));
        Vector vtAB = vA.add(vB.multiply(-1));
        
        float j = (-(1+e)*vAB.dot(MTD))/(MTD.dot(MTD)*(invMassA+invMassB)+pow(rA.normal().dot(MTD),2)*A.invMomentOfInertia+pow(rB.normal().dot(MTD),2)*B.invMomentOfInertia);

        if (invMassA != 0) A.impulse(MTD, 1 - A.mass/(A.mass + B.mass), +j, rA);
        if (invMassB != 0) B.impulse(MTD, B.mass/(A.mass + B.mass) - 1, -j, rB);
      }
    }
  }
  
  /**
   * Finds the points of contact
   * between the two objects
   */
  public Point[] findContactPoints(PhysicsModel A, PhysicsModel B, Vector MTD)
  {
    Point[] R = new Point[2];
    R[0] = new Point();
    R[1] = new Point();
    
    Point[] supportA = findSupportPoints(A.poly, MTD);
    Point[] supportB = findSupportPoints(B.poly, MTD.multiply(-1));
    
    int sNumA = supportA.length;
    int sNumB = supportB.length;
    
    if (sNumA == 1 && sNumB == 1)
    {
      R[0] = supportA[0];
      R[1] = supportB[0];
    }
    else if (sNumA == 1 && sNumB == 2)
    {
      R[0] = supportA[0];
      Vector edge = new Vector(supportB[1].x-supportB[0].x, supportB[1].y-supportB[0].y);
      R[1] = new Point(supportA[0].x + A.center.x - B.center.x, supportA[0].y + A.center.y - B.center.y);
    }
    else if (sNumA == 2 && sNumB == 1)
    {
      R[1] = supportB[0];
      Vector edge = new Vector(supportA[1].x-supportA[0].x, supportA[1].y-supportA[0].y);
      R[0] = new Point(supportB[0].x + B.center.x - A.center.x, supportB[0].y + B.center.y - A.center.y);
    }
    
    return R;
  }
  
  /**
   * Finds the support points of the two objects
   * which is then used to calculate the contact
   * points
   */
  public Point[] findSupportPoints(Polygon P, Vector V)
  {
    ArrayList R = new ArrayList(2);
    float minimumDot = new Vector(P.vertices[0]).dot(V);
    float[] dot = new float[P.vertices.length];
    dot[0] = minimumDot;
    
    for (int i=1; i<P.vertices.length; i++)
    {
      dot[i] = new Vector(P.vertices[i]).dot(V);
      if (dot[i] < minimumDot) minimumDot = dot[i];
    }

    final float threshold = 5E-10f;

    for (int i=0; i<P.vertices.length; i++)
    {
      if (dot[i]-minimumDot < threshold)
      {
        R.add(P.vertices[i]);
        if (R.size() == 2) break;
      }
    }
    
    R.trimToSize();

    return (Point[]) R.toArray(new Point[R.size()]);
  }
  
  /**
   * Calculates the Mean 
   * Transitional Distance (?)
   * vector
   */
  public Vector calculateMTD(PhysicsModel A, PhysicsModel B)
  {
    Vector newPushVector = new Vector();
    ArrayList pushVectors = new ArrayList();

    for (int j=0; j<A.poly.numEdges(); j++)
    {
      float intervalIntersection = calculateIntervalIntersection(A.poly.getEdge(j).normal(), A, B);
      if (intervalIntersection == 0) return new Vector();
      newPushVector = new Vector();
      newPushVector.set(intervalIntersection, A.poly.getEdge(j).normal().dir());
      pushVectors.add(newPushVector);
    }
    for (int j=0; j<B.poly.numEdges(); j++)
    {       
      float intervalIntersection = calculateIntervalIntersection(B.poly.getEdge(j).normal(), A, B);
      if (intervalIntersection == 0) return new Vector();
      newPushVector = new Vector();
      newPushVector.set(intervalIntersection, B.poly.getEdge(j).normal().dir());
      pushVectors.add(newPushVector);
    }

    Vector currentBest = new Vector();

    for (int i=0; i<pushVectors.size(); i++)
    {
      Vector pushVector = (Vector) pushVectors.get(i);
            
      if ((pushVector.length() < currentBest.length()) || currentBest.length() == 0)
      {
        currentBest = pushVector;
      }
    }

    Vector D = new Vector();
    D.a = A.center.x - B.center.x;
    D.b = A.center.y - B.center.y;
    
    if (D.dot(currentBest) < 0) currentBest = currentBest.multiply(-1);

    return currentBest; 
  }
  
  /**
   * Calculates the intersecting
   * interval of two objects in
   * a certain direction
   */
  public float calculateIntervalIntersection(Vector N, PhysicsModel A, PhysicsModel B)
  {
    Interval intervalA = calculateInterval(N,A);
    Interval intervalB = calculateInterval(N,B);
    Interval intersection = intervalA.intersect(intervalB); 
    return intersection.length();
  }
  
  /**
   * Projects the object onto an axis
   */
  public Interval calculateInterval(Vector axis, PhysicsModel O)
  { 
    Interval R = new Interval(0,0);
    for (int i=0; i<O.poly.numEdges(); i++)
    {
      R.insert((O.poly.getEdge(i).dot(axis) + new Vector(O.poly.vertices[i]).dot(axis))/axis.length());
    }
    return R.offset((new Vector(O.center).dot(axis))/axis.length());
  } 
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// HumanAgent.java
// The agents controlled by the human player.
// ------------------------------------------------------------------------------

class HumanAgent extends Agent
{
  /**
   * Constructor
   */
  HumanAgent(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, P, M, C, R, S, I);
  }
  
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    reset();
    
    drive(keys[0], keys[1], keys[2], keys[3], "Human");
    
    move();
    damp(dampK/100,0.9f);
    return this;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Interval.java
// Data type for simple 1 dimensional hyper-planes.
// ------------------------------------------------------------------------------

class Interval
{
  float min;
  float max;
    
  /**
   * Constructor
   */
  Interval(float Min, float Max)
  {
    min = Min;
    max = Max;
  }
  
  /**
   * Update the interval
   */
  public void insert(float value)
  {
    if (value > max)
    {
      max = value;
    }
    else if (value < min)
    {
      min = value;
    }
  }
  
  /**
   * Offset the interval
   */
  public Interval offset(float c)
  {
    Interval R = new Interval(0,0);
    R.min = min + c;
    R.max = max + c;
    return R;
  }
  
  /**
   * Find the intersection of two intervals
   */
  public Interval intersect(Interval I)
  {
    Interval R;
    
    if ((min <= I.max && min >= I.min) || (I.min <= max && I.min >= min))
    {
        R = new Interval(max(min, I.min),min(max, I.max));
    }
    else
    {
      R = new Interval(0,0);
    }
    
    return R;
  }
  
  /**
   * Length of the interval
   */
  public float length()
  {
    return max-min;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Material.java
// Every object in the game world will have a certain material type. This will define
// colour and specular shine.
// ------------------------------------------------------------------------------

class Material
{
  float specular;              // The amount of shine the object has (0-100)
  float glow;                  // How much the object glows (0-100)
  int body;                  // The object's body's color
  int border;                // The object's border's color
  int stroke;                  // The border's thickness
  // -- Physics stuff ------
  float invDensity;            // 1/density of the material
  float bounce;                // Bounce constant
  
  /**
   * Constructor
   */
  Material(float Specular, float Glow, int Body, int Border, int Stroke, float InvDensity, float Bounce)
  {
    specular = Specular;
    glow = Glow;
    body = Body;
    border = Border;
    stroke = Stroke;
    invDensity = InvDensity;
    bounce = Bounce;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Model.java
// An abstract type for everything that exists in the game world. Characters, obstacles
// etc etc will all have to extend this abstract class and implement the abstract
// methods.
// ------------------------------------------------------------------------------

abstract class Model
{
  String name;
  String type;
  Polygon poly;
  Material material;
  Point center;
  Settings settings;
  ParticleManager particleManager;
  Stack toBeAdded;
  
  /**
   * Constructor
   */
  Model(String N, String T, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    name = N;
    type = T;
    poly = P.rotate(R);
    material = M;
    center = C;
    settings = S;
    particleManager = new ParticleManager();
    toBeAdded = new Stack();
    
    parseInput(I);
  }

  /**
   * Parses the input string into a
   * nice and readable array
   */
  public String[] inputToArray(String input)
  {
    String[] R = input.split("&");
    for (int i=0; i<R.length; i++)
    {
      R[i] = R[i].split("=")[1];
    }
    return R;
  }
  
  /**
   * Convert a String to an Integer
   */
  public int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  public float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
  
  /**
   * Creates and destroys particles at correct
   * intervals with changes in rotation and speed
   */
  public void emitParticles(String type, float rotation, Point center, Vector veloc, int number)
  {
    for (int i=0; i<number; i++)
    {
      Particle P;
      float rand = random(0.99f,1.01f);    // We need this to make each particle slightly unique
      
      if (type.equals("AccelHuman"))
      {
        P = new AccelHumanParticle(rotation, center, veloc.multiply(rand));
      }
      else
      {
        break;
      }
      
      particleManager.addParticle(P);
    }
  }
  
  /**
   * Adds an item to the list of 
   * models to be added to the world
   */
  public void addModel(Model M)
  {
    toBeAdded.push(M);
  }
  
  /**
   * Returns the model that it 
   * wants to add to the gameworld.
   *
   * Only one model can be added by
   * any model at each frame.
   */
  public Model modelToBeAdded()
  {
    if (toBeAdded.size() != 0)
    {
      return ((Model) toBeAdded.pop());
    }
    else
    {
      return null;
    }
  }
  
  /**
   * Give the agent that 'runs' the model
   * the information it needs to update itself
   * in this frame.
   */
  public abstract Model frame(ArrayList models, boolean[] keys);
  
  /**
   * Each model should be able to
   * parse its settings string from
   * the game world.
   */
  public abstract void parseInput(String inputs);
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Particle.java
// An abstract type for the special effects in the game world. This includes smoke,
// sparks, flashes, etc.
// ------------------------------------------------------------------------------

abstract class Particle
{
  float rotation;
  Point center;
  Vector veloc;
  Vector accel;
  boolean dead;
  boolean renderOnTop;
  
  /**
   * Constructor
   */
  Particle(float R, Point C, Vector V, boolean O)
  {
    rotation = R;
    center = C;
    veloc = V;
    accel = new Vector();
    renderOnTop = O;
  }
  
  /**
   * Kills the particle
   */
  public void die()
  {
    dead = true;
  }
  
  /**
   * Each particle should be able to
   * update its position after each frame
   */
  public abstract void frame();
  
  /**
   * Each particle should be able to
   * draw itself
   */
  public abstract void render(float x, float y);
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// ParticleManager.java
// Manages the particle updates for each model (its a queue!).
// ------------------------------------------------------------------------------

class ParticleManager
{
  ArrayList particles;
  
  /**
   * Constructor
   */
  ParticleManager()
  {
    particles = new ArrayList();
  }
  
  /**
   * Add a particle to the queue
   */
  public void addParticle(Particle P)
  {
    particles.add(P);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// PhysicsModel.java
// A more sophisticated version of Model, this agent has built in physics
// attributes and functions.
// ------------------------------------------------------------------------------

class PhysicsModel extends Model
{
  ArrayList forces;
  Vector veloc;
  Vector accel;
  
  float torque;
  float invMomentOfInertia;
  float rotationSpeed;
  float mass;
   
  /**
   * Constructor
   */
  PhysicsModel(String N, String T, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, T, P, M, C, R, S, I);
    
    forces = new ArrayList();
    veloc = new Vector();
    accel = new Vector();
    
    torque = 0;
    invMomentOfInertia = calculateMoment();
    rotationSpeed = 0;
    mass = poly.area()/material.invDensity;
  }
   
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    return this;
  }
  
  /**
   * Moves the object correctly
   */
  public void move()
  {
    float delta_t = 1/frameRate;
    
    // ---------------------------
    // Calculate total force on object
    // ---------------------------
    
    Vector totalForceVector = new Vector();
    for (int i=0; i<forces.size(); i++)
    {
      totalForceVector = totalForceVector.add(((Force) forces.get(i)).vector);
    }
    
    // --------------------------
    // Translational movement
    // --------------------------
    
    accel = totalForceVector.multiply(1/mass);
    center = ((veloc.multiply(delta_t)).add(accel.multiply(pow(delta_t,2)/2))).add(center);
    veloc = veloc.add(accel.multiply(delta_t));
    
    // --------------------------
    // Rotational movement
    // --------------------------
    
    for (int i=0; i<forces.size(); i++)
    {
      Force f = ((Force) forces.get(i));
      Vector r = new Vector(f.point);
      torque += f.vector.normal().dot(r);
    }
    
    torque *= invMomentOfInertia;
    float rotationAmount = rotationSpeed*delta_t+torque*(pow(delta_t,2)/2);
    if (abs(rotationAmount) > radians(0.1f)) poly = poly.rotate(rotationAmount);
    rotationSpeed += torque*delta_t;
  }
  
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {
  }
  
  /**
   * Calculates the moment of inertia of
   * this model's polygon
   */
  public float calculateMoment()
  {
    float R = 0;
    
    if ( material.invDensity != 0 )
    {
      float temp = 0;
      
      for (int i=0; i<poly.numVertex(); i++)
      {
        Vector Pn;
        Vector Pnp1;
        
        Pn = new Vector(poly.vertices[i%poly.numVertex()]);
        Pnp1 = new Vector(poly.vertices[(i+1)%poly.numVertex()]);
        
        temp += (abs(Pn.dot(Pnp1.normal()))*(Pn.dot(Pn)+Pnp1.dot(Pnp1)+Pn.dot(Pnp1)));
      }
      
      R = 1/(temp/material.invDensity/12);
    }
    
    return R;
  }
  
  /**
   * Resets values to get ready for the 
   * next frame
   */
  public void reset()
  {
    forces = new ArrayList();
    accel = new Vector();
    torque = 0; 
  }
  
  /**
   * Creates an impulse on the object
   * when it collides with something else
   */  
  public void impulse(Vector v, float k, float j, Vector r)
  { 
    float invMass = 1/mass;
    
    center = v.multiply(k).add(center);
    veloc = veloc.add(v.multiply(j*invMass));
    rotationSpeed -= v.multiply(j).dot(r.normal())*invMomentOfInertia;
  }
  
  /**
   * Slows down the movement and rotation
   * artificially
   */
  public void damp(float kv, float kr)
  {
    veloc = veloc.multiply(kv);
    rotationSpeed *= kr;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Point.java
// Pretty self explanatory I would say. No?
// ------------------------------------------------------------------------------

class Point
{
  float x;
  float y;
  
  /**
   * Constructor with no arguments
   */
  Point()
  {
    x = 0;
    y = 0;
  }
  
  /**
   * Constructor with set x and y values
   */
  Point(float X, float Y)
  {
    x = X;
    y = Y;
  }
  
  /**
   * Rotates the point by theta radians
   */
  public Point rotate(float theta)
  {
    Point R = new Point();
    float xprev = x;
    float yprev = y;
    R.x = cos(theta)*xprev+sin(theta)*yprev;
    R.y = -sin(theta)*xprev+cos(theta)*yprev;
    return R;
  }
  
  /**
   * Multiply by k
   */
  public Point multiply(float k)
  {
    Point R = new Point();
    R.x = x*k;
    R.y = y*k;
    return R;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Polygon.java
// Data structure for storing polygons.
// ------------------------------------------------------------------------------

class Polygon
{
  Point[] vertices;
  float rotation;
  float radius;
  
  /**
   * Constructor with no arguments
   */
  Polygon()
  {
    vertices = new Point[0];
    rotation = 0;
  }
  
  /**
   * Constructor with set vertices
   */
  Polygon(Point[] Vertices)
  {
    vertices = Vertices;
    rotation = 0;
  }
  
  /**
   * Returns the number of vertices in
   * the polygon
   */
  public int numVertex()
  {
    return vertices.length;
  }
  
  /**
   * Adds a vertex to the current vertex
   * list
   */
  public void insertVertex(Point V)
  {
    Point[] verticesNew = new Point[numVertex()+1];
    
    for (int i=0; i<numVertex(); i++)
    {
      verticesNew[i] = vertices[i];
    }
    
    verticesNew[numVertex()] = V;
    vertices = verticesNew;
    
    float distance = sqrt(pow(V.x,2)+pow(V.y,2));
    if (distance > radius) radius = distance;
  }
  
  /**
   * Calculates the area of the polygon
   */
  public float area()
  {
    float R = 0;
    for (int i=0; i<numVertex(); i++)
    {
      R += (vertices[i%numVertex()].x*vertices[(i+1)%numVertex()].y-vertices[(i+1)%numVertex()].x*vertices[i%numVertex()].y)/2;
    }
    return abs(R);
  }
  
  /**
   * Rotates the polygon
   */
  public Polygon rotate(float theta)
  {
    Polygon R = new Polygon();
    
    R.rotation = rotation + theta;
    if (R.rotation > TWO_PI) R.rotation -= TWO_PI;
    if (R.rotation < 0 ) R.rotation += TWO_PI;
    
    for (int i=0; i<numVertex(); i++)
    {
      R.insertVertex(vertices[i].rotate(theta));
    }
    
    return R;
  }
  
  /**
   * Returns the number of edges
   * the polygon has
   */
  public int numEdges()
  {
    return vertices.length;
  }
  
  /**
   * Returns a vector corresponding
   * to the correct edge
   */
  public Vector getEdge(int index)
  {
    return new Vector(vertices[(index+1)%numEdges()].x-vertices[index%numEdges()].x,vertices[(index+1)%numEdges()].y-vertices[index%numEdges()].y);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Renderer.java
// An instance of this class will take various inputs from the game engine
// such as the models, the light sources, etc. and display them on the screen.
// ------------------------------------------------------------------------------

class Renderer
{
  Settings settings;
  TrigTable T;
  
  /**
   * Constructor
   */
  Renderer(Settings S)
  {
    settings = S;
    T = new TrigTable(1);
  }
  
  /**
   * Draws the current world onto screen
   */
  public void frame(GameWorld gameWorld)
  {
    drawBackground();
    drawModels(gameWorld.models);
  }
  
  /**
   * Draws each model with the correct colours
   */
  public void drawModels(ArrayList models)
  {
    // ------------------------------
    // Finding the camera
    // ------------------------------
      
    Model camera = findCamera(models);
    
    for (int i=models.size()-1; i>=0; i--)
    {
      Model model = (Model) models.get(i);
      
      // ------------------------------
      // Layer 1: Particles not on top
      // This section could be redone.
      // ------------------------------

      for (int j=0; j<model.particleManager.particles.size(); j++)
      {
        Particle particle = (Particle) model.particleManager.particles.get(j);
      
        if (!particle.dead && !particle.renderOnTop)
        {
          particle.frame();
          particle.render(settings.PIXELS_PER_METER*(particle.center.x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(particle.center.y - camera.center.y) + height/2);
          model.particleManager.particles.set(j,particle);
        }
      }
        
      // ------------------------------
      // Layer 2: Body Fills
      // ------------------------------
      
      fill(model.material.body);
      noStroke();
      
      beginShape(POLYGON);
      for (int j=0; j<model.poly.numVertex(); j++)
      {
        float x = model.poly.vertices[j].x;
        float y = model.poly.vertices[j].y;
        vertex(settings.PIXELS_PER_METER*(model.center.x + x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(model.center.y + y - camera.center.y) + height/2); 
      }
      endShape();
          
      // ------------------------------
      // Layer 3: Specular Shine
      // ------------------------------
      
      if(model.material.specular != 0)
      {
        for (int j=0; j<models.size(); j++)
        {
          Model glowModel = (Model) models.get(j);
          
          if (j != i && glowModel.material.glow != 0)
            {          
            float distance = pow(model.center.x - glowModel.center.x,2) + pow(model.center.y - glowModel.center.y,2);
            float[] distances = new float[model.poly.numVertex()];
            int kMax = 0;
            float kMaxVal = 0;
          
            // Fill up the distances array from the center of model 
            // to each vertex.
            for (int k=0; k<model.poly.numVertex(); k++)
            {
              distances[k] = pow(glowModel.center.x - model.poly.vertices[k].x - model.center.x,2) + pow(glowModel.center.y - model.poly.vertices[k].y - model.center.y,2);
              
              if ( distances[k] > kMaxVal)
              {
                kMax = k;
                kMaxVal = distances[k];
              }
            }
            
            // Now draw the specular shine
            noStroke();
            fill(255,255,255,((float) glowModel.material.glow/100)*((float) model.material.specular/100)*255);
            
            beginShape(POLYGON);
            for (int k=0; k<distances.length+1; k++)
            {
              float x = model.poly.vertices[(kMax+k)%distances.length].x;
              float y = model.poly.vertices[(kMax+k)%distances.length].y;
              vertex(settings.PIXELS_PER_METER*(model.center.x + x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(model.center.y + y - camera.center.y) + height/2);
            }
            for (int k=distances.length; k>=0; k--)
            {
              if ((kMax+k)%distances.length != kMax)
              {
                float amp = distances[(kMax+k)%distances.length]/distances[kMax];
                if ( amp > 0.95f ) amp = 1;
                if ( amp < settings.MAX_SPECULAR/100 ) amp = settings.MAX_SPECULAR/100;
                float x = model.poly.vertices[(kMax+k)%distances.length].x*amp;
                float y = model.poly.vertices[(kMax+k)%distances.length].y*amp;
                vertex(settings.PIXELS_PER_METER*(model.center.x + x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(model.center.y + y - camera.center.y) + height/2);
              }
            }
            endShape();
          }
        }
      } 
      
      // ------------------------------
      // Layer 4: Borders
      // ------------------------------
      
      if (model.material.stroke != 0)
      {
        noFill();
        stroke(model.material.border);
        strokeWeight(model.material.stroke);
      
        beginShape();
        for (int j=0; j<model.poly.numVertex(); j++)
        {
          float x = model.poly.vertices[j].x;
          float y = model.poly.vertices[j].y;
          vertex(settings.PIXELS_PER_METER*(model.center.x + x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(model.center.y + y - camera.center.y) + height/2); 
        }
        endShape(CLOSE); 
      } 
      
      // ------------------------------
      // Layer 5: Particles on top
      // This section could be redone.
      // ------------------------------

      for (int j=0; j<model.particleManager.particles.size(); j++)
      {
        Particle particle = (Particle) model.particleManager.particles.get(j);
      
        if (!particle.dead && particle.renderOnTop)
        {
          particle.frame();
          particle.render(settings.PIXELS_PER_METER*(particle.center.x - camera.center.x) + width/2, settings.PIXELS_PER_METER*(particle.center.y - camera.center.y) + height/2);
          model.particleManager.particles.set(j,particle);
        }
      }
    }
  }
  
  /**
   * Finds the camera in the list of models
   */
  public Model findCamera(ArrayList models)
  {
    for (int i=0; i<models.size(); i++)
    {
      Model model = (Model) models.get(i);
      
      if (model.name.equals("camera"))
      {
        return model;
      }
    }
    
    return null;
  }
  
  /**
   * Draws everything related to the background
   */
  public void drawBackground()
  {
    background(255,255,255);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Settings.java
// Loads the game's settings from file and stores them in a handy class
// ------------------------------------------------------------------------------

class Settings
{
  // - The Settings Values --------
  String GAMEWORLD_PATH;
  float PIXELS_PER_METER;
  float MAX_SPECULAR;
  
  /**
   * Constructor
   * Loads the file and sets up the values
   */
  Settings(String path)
  {
    // Number of lines reserved for instructions
    int offset = 6;
    
    // Load the file
    String[] data = loadStrings(path);
    
    // Parse the file to find the values
    // on the right hand side of the =
    for (int i=offset; i<data.length; i++)
    {
      String[] tokens = data[i].split(" = ");
      data[i] = tokens[1];
    }
    
    // Instantiate the variables
    GAMEWORLD_PATH =                     data[offset+0];
    PIXELS_PER_METER =                   Str2Flt(data[offset+1]);
    MAX_SPECULAR =                       Str2Flt(data[offset+2]);
  }
  
  /**
   * Convert a String to an Integer
   */
  public int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  public float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Sheep.java
// The sheep that the human plays with.
// ------------------------------------------------------------------------------

class Sheep extends PhysicsModel
{
  float initVelocRange;
  float neighbourRadius;
  float separateK;
  float stickK;
  float neighbourSeparateRadius;
  
  Vector steer;
  
  /**
   * Constructor
   */
  Sheep(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Sheep", P, M, C, R, S, I);
    
    veloc = new Vector(random(-initVelocRange/2,initVelocRange/2), random(-initVelocRange/2,initVelocRange/2));
    steer = new Vector();
  }
  
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    reset();
        
    ArrayList neighbours = findNeighbours(models);
    
    if (neighbours.size() > 0) {
      // align(neighbours);
      stick(neighbours);
      separate(neighbours);
      escape(neighbours);
      wander();
    }
    else {
      follow(models);
    }
    
    steer();
    move();
    damp(0.98f, 0.90f);
    return this;
  }
  
  /**
   * Updates the sheep's velocity using
   * the steer vector
   */
  public void steer()
  {
    Point head = new Point(0,10).rotate(poly.rotation);
    Force steerForce = new Force(head, steer);
    forces.add(steerForce);
  }
  
  /**
   * Finds its neighbours in the list of models
   * in the world. Ideally has to be very fast.
   */
  public ArrayList findNeighbours(ArrayList models)
  {
    ArrayList R = new ArrayList();
    
    for (int i=0; i<models.size(); i++)
    {
      Model model = (Model) models.get(i);
      float d2 = pow(center.x-model.center.x,2)+pow(center.y-model.center.y,2);
      
      if ((model.name.equals("sheep") || model.name.equals("humanagent")) && d2 != 0 && d2 < pow(neighbourRadius,2))
      {
        R.add(model);
      }
    }
    
    return R;
  }
  
  /**
   * Tries to avoid colliding with any of its neighbours
   */
  public void separate(ArrayList models)
  {
    if (models.size() != 0)
    {
      int count = 0;
      
      // Find the mean center of all the
      // points in its neighbourhood
      Point meanCenter = new Point();
      for (int i=0; i<models.size(); i++)
      {  
        Model model = (Model) models.get(i);
        float d2 = pow(center.x-model.center.x,2)+pow(center.y-model.center.y,2);
        
        if (d2 < pow(neighbourSeparateRadius,2)) 
        {
          meanCenter = new Point(meanCenter.x + model.center.x, meanCenter.y + model.center.y);
          count++;
        }
      }
      
      if (count != 0)
      {
        meanCenter = meanCenter.multiply(1/((float) count));
      
        // RUN AWAY!
        Vector d = new Vector(center.x-meanCenter.x, center.y-meanCenter.y);
        steer = steer.add(d.multiply(separateK/100));
      }      
    }
  }
  
  /**
   * Run from the human agent
   */
  public void escape(ArrayList models)
  {
    if (models.size() != 0)
    {
      for (int i=0; i<models.size(); i++)
      {  
        Model model = (Model) models.get(i);
        
        if (model.name.equals("humanagent"))
        {
          // RUN AWAY!
          Vector d = new Vector(center.x-model.center.x, center.y-model.center.y);
          steer = steer.add(d.multiply(0.1f));   
        }
      }
    }
  }
  
  /**
   * Follow the human agent
   */
  public void follow(ArrayList models)
  {
    if (models.size() != 0)
    {
      for (int i=0; i<models.size(); i++)
      {  
        Model model = (Model) models.get(i);
        
        if (model.name.equals("humanagent"))
        {
          // Follow the agent
          Vector d = new Vector(center.x-model.center.x, center.y-model.center.y);
          steer = steer.add(d.multiply(-0.02f));   
        }
      }
    }
  }
  
  /**
   * Makes the sheep stick to their neighbours
   */
  public void stick(ArrayList models)
  {
    if (models.size() != 0)
    {
      // Find the mean center of all the
      // points in its neighbourhood
      Point meanCenter = new Point();
      for (int i=0; i<models.size(); i++)
      {  
        Model model = (Model) models.get(i);
        meanCenter = new Point(meanCenter.x + model.center.x, meanCenter.y + model.center.y);
      }
      meanCenter = meanCenter.multiply(1/((float) models.size()));
      
      // STICK!
      Vector d = new Vector(meanCenter.x-center.x, meanCenter.y-center.y);
      steer = steer.add(d.multiply(stickK/100));
    }
  }
  
  /**
   * Just wander around the map
   */
  public void wander()
  {
    Vector deviation = veloc.normal().multiply(random(-0.1f,0.1f));
    steer = steer.add(veloc.add(deviation).multiply(0.035f));
  }
   
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {
    String[] args = inputToArray(input);
     
    initVelocRange = Str2Int(args[0]);
    neighbourRadius = Str2Flt(args[1]);
    neighbourSeparateRadius = Str2Flt(args[2]);
    separateK = Str2Flt(args[3]);
    stickK = Str2Flt(args[4]);
  }
  
  /**
   * Resets values to get ready for the 
   * next frame
   */
  public void reset()
  {
    steer = new Vector();
    forces = new ArrayList();
    accel = new Vector();
    torque = 0; 
  }
}
  

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Sheep.java
// The sheep that the human plays with.
// ------------------------------------------------------------------------------

class SheepDispenser extends Model
{
  int frame;
  int numSheepCurrent;
  int numSheep;
  int sheepDelay;
  Polygon sheepPolygon;
  Material sheepMaterial;
  
  /**
   * Constructor
   */
  SheepDispenser(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "SheepDispenser", P, M, C, R, S, I);
    
    DataLoader dataLoader = new DataLoader();
    sheepPolygon = dataLoader.loadPolygon("tinyBox");
    sheepMaterial = dataLoader.loadMaterial("wool");
  }
  
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    frame++;
    
    if (numSheepCurrent < numSheep && frame%sheepDelay == 0)
    {
      addModel(new Sheep("sheep", sheepPolygon, sheepMaterial, center, 0, settings, "initialVelocRange=50&neighbourRadius=50&neighbourSeparateRadius=20&separateK=35&stickK=6&wanderK=20"));
      numSheepCurrent++;
    }
    
    return this;
  }
   
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {
    String[] args = inputToArray(input);
     
    numSheep = Str2Int(args[0]);
    sheepDelay = Str2Int(args[1]);
  }
}
  

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Static.java
// Static object that does nothing.
// ------------------------------------------------------------------------------

class Static extends PhysicsModel
{
  /**
   * Constructor
   */
  Static(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Static", P, M, C, R, S, I);
  }
   
  /**
   * Executes one frame
   */
  public Model frame(ArrayList models, boolean[] keys)
  {
    move();
    damp(0.99f, 0.98f);
    return this;
  }
   
  /**
   * Parse the input string
   */
  public void parseInput(String input)
  {
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// TrigTable.java
// Precomputes various trigonometric values and stores them in a table. This
// will increase trig-call performance by around 300 to 400 percent.
// ------------------------------------------------------------------------------

class TrigTable
{
  int numSeg;            // The number of segments a single degree has  
  float[] sinTable;            // The table for sin()s of different angles
  float[] cosTable;            // The table for cos()s of different angles
  float[] tanTable;            // The table for tan()s of different angles
  
  /**
   * Constructor
   */
  TrigTable(int N)
  {
    numSeg = max(N, 1);    // Makes sure numSeg isn't lower than 1
    
    // Define tables
    sinTable = new float[360*numSeg];
    cosTable = new float[360*numSeg];
    tanTable = new float[360*numSeg];
    
    // Fill the table with the correct values
    for (int i=0; i<360*numSeg; i++)
    {
      float angle = i/((float) numSeg);
      sinTable[i] = (float) Math.sin(radians(angle));
      cosTable[i] = (float) Math.cos(radians(angle));
      tanTable[i] = (float) Math.tan(radians(angle));
    }
  }
  
  /**
   * Finds and returns the sin of the angle
   */
  public float sin(float angle)
  {
    return sinTable[nice(angle)];
  }
  
  /**
   * Finds and returns the cos of the angle
   */
  public float cos(float angle)
  {
    return cosTable[nice(angle)];
  }
  
  /**
   * Finds and returns the tan of the angle
   */
  public float tan(float angle)
  {
    return tanTable[nice(angle)];
  }
  
  /**
   * Takes a radians value and gets the
   * appropriate index for the TrigTable
   * Kudos to seltar.wliia.org
   */
  public int nice(float angle)
  {
    // Convert to degrees
    int deg = round(degrees(angle));
    
    // Get the angle between 0 and 360
    while(deg < 0)     deg += 360;
    while(deg >= 360)  deg -= 360;
    return deg*numSeg;
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Vector.java
// Pretty self explanatory I would say. No?
// ------------------------------------------------------------------------------

class Vector
{
  float a;
  float b;
  
  /**
   * Constructor with no arguments
   */
  Vector()
  {
    a = 0;
    b = 0;
  }
  
  /**
   * Constructor with set a and b values
   */
  Vector(float A, float B)
  {
    a = A;
    b = B;
  }
  
  /**
   * Constructor corresponding to point
   */
  Vector(Point P)
  {
    a = P.x;
    b = P.y;
  }
  
  /**
   * Returns the length of the vector
   */
  public float length()
  {
    return sqrt(pow(a,2)+pow(b,2));
  } 
  
  /**
   * Returns the angle of the vector relative
   * to the positive x axis
   */
  public float dir()
  {
    return atan2(b,a);
  } 
  
  /**
   * Returns the vector, scaled by k
   */
  public Vector multiply(float k)
  {
    return new Vector(k*a, k*b);
  }
  
  /**
   * Returns the vector plus the vector V
   */
  public Vector add(Vector V)
  {
    return new Vector(a+V.a, b+V.b);
  } 
  
  /**
   * Returns the vector plus the point P
   */
  public Point add(Point P)
  {
    return new Point(a+P.x, b+P.y);
  } 
  
  /**
   * Returns the vector dot (scalar) the vector V
   */
  public float dot(Vector V)
  {
    return (a*V.a + b*V.b);
  } 
  
  /**
   * Returns the normal to this vector
   */
  public Vector normal()
  {
    return new Vector(-b,a);
  }
  
  /**
   * Sets up the vector in the correct length
   * and direction
   */
  public void set(float L, float D)
  {
    a = L*cos(D);
    b = L*sin(D);
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "Arkigen" });}}