// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Camera.java
// Agent that controls the movements of the camera.
// ------------------------------------------------------------------------------

class Camera extends Model
{
  int follow;
  float invSpeed;
  float maxDisp;
  
  /**
   * Constructor
   */
  Camera(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Camera", P, M, C, R, S, I);
  }
  
   /**
   * Executes one frame
   */
  Model frame(ArrayList models, boolean[] keys)
  {
    if (invSpeed != 0)
    {
      Model followModel = (Model) models.get(follow);
      Vector difference = new Vector(followModel.center.x - center.x, followModel.center.y - center.y);
      Point targetCenter = difference.multiply(1/invSpeed).add(center);
    
      if (new Vector(targetCenter).length() > maxDisp)
      {
        float theta = atan2(targetCenter.y, targetCenter.x);
        float x = maxDisp*cos(theta);  // Can't use TrigTable because it 
        float y = maxDisp*sin(theta);  // isn't smooth enough for the camera
        center = new Point(x,y);
      }
      else
      {
        center = targetCenter;
      }
    }
    
    return this;
  }
  
  /**
   * Parse the input string
   */
  void parseInput(String input)
  {     
    String[] args = inputToArray(input);
     
    follow = Str2Int(args[0]);
    invSpeed = Str2Flt(args[1]);
    maxDisp = Str2Flt(args[2]);
  }
}
