// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// TrigTable.java
// Precomputes various trigonometric values and stores them in a table. This
// will increase trig-call performance by around 300 to 400 percent.
// ------------------------------------------------------------------------------

class TrigTable
{
  int numSeg;            // The number of segments a single degree has  
  float[] sinTable;            // The table for sin()s of different angles
  float[] cosTable;            // The table for cos()s of different angles
  float[] tanTable;            // The table for tan()s of different angles
  
  /**
   * Constructor
   */
  TrigTable(int N)
  {
    numSeg = max(N, 1);    // Makes sure numSeg isn't lower than 1
    
    // Define tables
    sinTable = new float[360*numSeg];
    cosTable = new float[360*numSeg];
    tanTable = new float[360*numSeg];
    
    // Fill the table with the correct values
    for (int i=0; i<360*numSeg; i++)
    {
      float angle = i/((float) numSeg);
      sinTable[i] = (float) Math.sin(radians(angle));
      cosTable[i] = (float) Math.cos(radians(angle));
      tanTable[i] = (float) Math.tan(radians(angle));
    }
  }
  
  /**
   * Finds and returns the sin of the angle
   */
  float sin(float angle)
  {
    return sinTable[nice(angle)];
  }
  
  /**
   * Finds and returns the cos of the angle
   */
  float cos(float angle)
  {
    return cosTable[nice(angle)];
  }
  
  /**
   * Finds and returns the tan of the angle
   */
  float tan(float angle)
  {
    return tanTable[nice(angle)];
  }
  
  /**
   * Takes a radians value and gets the
   * appropriate index for the TrigTable
   * Kudos to seltar.wliia.org
   */
  int nice(float angle)
  {
    // Convert to degrees
    int deg = round(degrees(angle));
    
    // Get the angle between 0 and 360
    while(deg < 0)     deg += 360;
    while(deg >= 360)  deg -= 360;
    return deg*numSeg;
  }
}
