// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Vector.java
// Pretty self explanatory I would say. No?
// ------------------------------------------------------------------------------

class Vector
{
  float a;
  float b;
  
  /**
   * Constructor with no arguments
   */
  Vector()
  {
    a = 0;
    b = 0;
  }
  
  /**
   * Constructor with set a and b values
   */
  Vector(float A, float B)
  {
    a = A;
    b = B;
  }
  
  /**
   * Constructor corresponding to point
   */
  Vector(Point P)
  {
    a = P.x;
    b = P.y;
  }
  
  /**
   * Returns the length of the vector
   */
  float length()
  {
    return sqrt(pow(a,2)+pow(b,2));
  } 
  
  /**
   * Returns the angle of the vector relative
   * to the positive x axis
   */
  float dir()
  {
    return atan2(b,a);
  } 
  
  /**
   * Returns the vector, scaled by k
   */
  Vector multiply(float k)
  {
    return new Vector(k*a, k*b);
  }
  
  /**
   * Returns the vector plus the vector V
   */
  Vector add(Vector V)
  {
    return new Vector(a+V.a, b+V.b);
  } 
  
  /**
   * Returns the vector plus the point P
   */
  Point add(Point P)
  {
    return new Point(a+P.x, b+P.y);
  } 
  
  /**
   * Returns the vector dot (scalar) the vector V
   */
  float dot(Vector V)
  {
    return (a*V.a + b*V.b);
  } 
  
  /**
   * Returns the normal to this vector
   */
  Vector normal()
  {
    return new Vector(-b,a);
  }
  
  /**
   * Sets up the vector in the correct length
   * and direction
   */
  void set(float L, float D)
  {
    a = L*cos(D);
    b = L*sin(D);
  }
}
